﻿
/**
 *	ShapeのEdge:Bezje
 */

package {
	
	import flash.geom.Point;
	import VectorPoint;
	
	public class EdgeBezje extends Object implements IEdge{
		
		//ベジェ曲線パラメータ
		private var x0:Number;
		private var y0:Number;
		private var x1:Number;	//アンカー
		private var y1:Number;	//アンカー
		private var x2:Number;
		private var y2:Number;
		
		private var length_:Number;
		
		/**
		 *	コンストラクタ
		 */
		public function EdgeBezje( _p0:Point , _p1:Point , _p2:Point ){
			setControls( _p0 , _p1 , _p2 );
		}
		
		
		/**
		 *	ポイントの更新
		 */
		public function setControls( _p0:Point , _p1:Point , _p2:Point ):void{
			x0 = _p0.x;
			y0 = _p0.y;
			x1 = _p1.x;
			y1 = _p1.y;
			x2 = _p2.x;
			y2 = _p2.y;
			
			length_ = Number.NaN;
		}
		
		public function get p0x():Number{
			return x0;
		}
		public function get p0y():Number{
			return y0;
		}
		public function get p1x():Number{
			return x1;
		}
		public function get p1y():Number{
			return y1;
		}
		public function get p2x():Number{
			return x2;
		}
		public function get p2y():Number{
			return y2;
		}
		
		
		/**
		 *	2次元ベジェ曲線上の位置を取得
		 *	@param ｔ(0<=t<=1)
		 */
		public function getPosAt( t:Number ):Point{
			var tp:Number = 1.0 - t;
			return new Point( x0*tp*tp + 2*x1*t*tp + x2*t*t , y0*tp*tp + 2*y1*t*tp + y2*t*t );
		}
		
		/**
		 *	2次元ベジェ曲線微分
		 *	@param ｔ(0<=t<=1)
		 */
		public function getDeffAt( t:Number ):Point{
			return new Point( 2*( t*( x0 + x2 - 2*x1 ) - x0 + x1 ) , 2*( t*( y0 + y2 - 2*y1 ) - y0 + y1 ) );
		}
		
		/**
		 *	線座標・ベクトル
		 *	@param ｔ(0<=t<=1)
		 */
		public function getVectorPointAt( t:Number ):VectorPoint{
			var tp:Number = 1.0 - t;
			return new VectorPoint( x0*tp*tp + 2*x1*t*tp + x2*t*t , y0*tp*tp + 2*y1*t*tp + y2*t*t , getDeffAt(t) );
		}
		
		
		/**
		 *	2次元ベジェ曲線の長さ
		 */
		public function get length():Number{
			return isNaN( length_ ) ? getLength() : length_;
		}
		
		
		/**
		 *	直線かどうか
		 */
		public function get isLine():Boolean{
			return false;
		}
		
		
		//------------------------------------------------------------
		
		
		/**
		 *	2次元ベジェ曲線の長さ
		 */
		private function getLength():Number{
			length_ = getLengthBezje2D();
			return length_;
		}
		
		private function getLengthBezje2D():Number{
			return ( integral(1.0) - integral(0.0) );
		}
		
		private function integral( t:Number ):Number{
			var kx:Number = x0 + x2 - 2 * x1;
			var ky:Number = y0 + y2 - 2 * y1;
			var ax:Number = - x0 + x1;
			var ay:Number = - y0 + y1;
			
			var XY:Number = kx*kx + ky*ky;
			var B:Number  = ( ax*kx + ay*ky )/XY;
			var C:Number  = ( ax*ax + ay*ay )/XY - B*B;
			
			var BT:Number  = B+t;
			var BTS:Number = Math.sqrt( BT*BT+C );
			
			return Math.sqrt(XY) * ( BTS*BT + C * Math.log( (BT + BTS)/Math.sqrt(C) ) );
		}
		
		/**
		 *	2次元ベジェ曲線の長さ:数値積分
		
		private function getLengthBezje2D2():Number{
			var len:Number = 0;
			var k:Number = 0.01;
			var p0:Point = new Point(x0,y0);
			for( var i:int=1;i<=100;i++ ){
				var t:Number = i*k;
				var tp:Number = 1.0 - t;
				var p1:Point = new Point( x0*tp*tp + 2*x1*t*tp + x2*t*t , y0*tp*tp + 2*y1*t*tp + y2*t*t );
				
				var pd:Point = p1.subtract(p0);
				len += pd.length;
				p0 = p1;
			}
			return len;
		}
		 */
		 
		public function toString():String{
			return "[bezje edge  x0=" +x0+ " y0=" +y0+ " x1=" +x1+ " y1=" +y1+ " x2=" +x2+ " y2=" +y2+ " ]";
		}
	}
	
}